
#include "createsshkeys.h"
#include <iostream>

#include "ksshagentview.h"
#include "settings.h"

#include <qlabel.h>
#include <qprocess.h>
#include <kconfigdialog.h>
#include <kcmdlineargs.h>
#include <kfiledialog.h>
#include <klistview.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <sys/wait.h>


KsshagentView::KsshagentView(QWidget *parent)
    : ksshagentview_base(parent)
{
    fileRedisplay();
    
    // Make a list of keys that are loaded
    QListViewItem * keyLoaded = keyList->firstChild();
    QStringList keysLoaded;
    while( keyLoaded ) {
        keysLoaded += keyLoaded->text(0);
        keyLoaded = keyLoaded->nextSibling();
    }
    
    // Parse command-line arguments
    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
    for(int i = 0; i < args->count(); i++) {
        // Add the key if not in keysLoaded
        if ( !keysLoaded.contains(args->arg(i)) ) {
            addKey(args->arg(i));
        }
    }
    args->clear();
    
    // Add keys configured to load on startup
    QStringList startupList = Settings::startupList();
    for ( QStringList::Iterator it = startupList.begin(); it != startupList.end(); ++it ) {
        // Add the key if not in keysLoaded
        if ( !keysLoaded.contains(*it) ) {
            addKey(*it);
        }
    }
    
    // Redisplay again to display keys that might be added above.
    // Maybe I should split up fileRedisplay.
    fileRedisplay();
}

KsshagentView::~KsshagentView()
{
    
}


/// SLOTS ///


// Not implemented yet
void KsshagentView::fileNew()
{
    //KDialog * foo = new CreateSSHKeys( this );
//     KDialog * dailog = new KDialog(this);
//     dailog->show();
    
    CreateSSHKeys *widget = new CreateSSHKeys;
        widget->show();
}


void KsshagentView::fileOpen()
{
    // Start filedialog in $HOME/.ssh
    QString startdir = QString(getenv("HOME"));
    startdir.append("/.ssh/");
    
    KURL keyFile = KFileDialog::getOpenURL(startdir);
    addKey(keyFile);
    redisplayKeys();
}


void KsshagentView::fileClose()
{
    if ( keyList->currentItem() ) {
        QString keyFile = keyList->currentItem()->text(0);
        closeKey(keyFile);
        redisplayKeys();
    }
}


void KsshagentView::fileRedisplay()
{
	redisplayKeys();
}


/// Functions ///


// Checks if the agent is running
void KsshagentView::checksshAgent()
{
    /*
	From ssh-add(1):

	"A unix-domain socket is created and the name of this socket is stored in the
	SSH_AUTH_SOCK environment variable."
	"The SSH_AGENT_PID environment variable holds the agent's process ID."
    */
    char * SSH_AGENT_PID = getenv("SSH_AGENT_PID");
    char * SSH_AUTH_SOCK = getenv("SSH_AUTH_SOCK");
    
    // Check if enviornment variables exist and 
    // send the "null" signal to check if ssh-agent is running
    if ( SSH_AGENT_PID && SSH_AUTH_SOCK && kill(atoi(SSH_AGENT_PID), 0) == 0) {
        // TODO: Check if socket is useable
    } else {
        KMessageBox::error( this,
            i18n("Please make sure ssh-agent is running before starting Ksshagent.\n"
            "See the documentation for more information."
            ), i18n("Could not find ssh-agent"));
	exit(1);
    }
}


// Refresh the keylist
void KsshagentView::redisplayKeys()
{
    QProcess * p = new QProcess();
    p->addArgument("ssh-add");
    p->addArgument("-l");
    p->setCommunication(QProcess::Stdout);
    p->start();
    
    int status;
    waitpid(p->processIdentifier(), &status, 0);
    
    keyList->clear();
    
    while ( p->canReadLineStdout() ) {
        QString line = QString(p->readLineStdout());
        if ( line == "The agent has no identities." ) {
            emit signalChangeStatusbar(line);
        } else {
            QString bits 		= line.section(' ', 0, 0);
            QString fingerprint 	= line.section(' ', 1, 1);
            QString file 		= line.section(' ', 2, 2);
            QString type 		= line.section(' ', 3, 3);
            type.remove("(").remove(")");
            new KListViewItem(keyList, file, type, bits, fingerprint);
        }
    }
}


// Add a key to ssh-agent
void KsshagentView::addKey(KURL keyFile)
{
    /*
	From ssh-add(1):
	
	"If ssh-add needs a passphrase, it will read the passphrase from the current terminal if it was run
	from a terminal. If ssh-add does not have a terminal associated with it but DISPLAY and SSH_ASKPASS
	are set, it will execute the program specified by SSH_ASKPASS and open an X11 window to read the
	passphrase.  This is particularly useful when calling ssh-add from a .xsession or related script.
	(Note that on some machines it may be necessary to redirect the input from /dev/null to make this
	work.)"
    */
    
    // Set custom path to ssh-askpass
    if (Settings::useCustomPath() == true) {
        setenv("SSH_ASKPASS",Settings::sshaskpassPath(),1);
    }
    
    // Run ssh-add and capture the output
    QProcess * p = new QProcess();
    p->addArgument("ssh-add");
    p->addArgument(keyFile.path());
    p->setCommunication(QProcess::Stderr);
    p->launch("");
    
    // Wait for process to finish
    int status;
    waitpid(p->processIdentifier(), &status, 0);
    
    emit signalChangeStatusbar(p->readLineStderr());
}


// Remove a key from the agent
void KsshagentView::closeKey(QString keyFile)
{
    // Run ssh-add and capture the output
    QProcess * p = new QProcess();
    p->addArgument("ssh-add");
    p->addArgument("-d");
    p->addArgument(keyFile);
    p->setCommunication(QProcess::Stderr);
    p->launch("");
    
    // Wait for process to finish
    int status;
    waitpid(p->processIdentifier(), &status, 0);
    
    emit signalChangeStatusbar(p->readLineStderr());
}


#include "ksshagentview.moc"

